/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "dbusservice.h"

#include "multitaskviewmanager.h"

namespace MultitaskView {

DBusService::DBusService(MultitaskViewManager *viewManager, QObject *parent)
    : QObject(parent),
      viewManager_(viewManager),
      connection_(QDBusConnection::sessionBus())
{
    startService();
}

DBusService::~DBusService()
{
    stopService();
}

bool DBusService::startService()
{
    return connection_.registerService(serviceName_)
            && connection_.registerObject(servicePath_, this, QDBusConnection::ExportAllSlots);
}

bool DBusService::stopService()
{
    connection_.unregisterObject(servicePath_);
    return connection_.unregisterService(serviceName_);
}

void DBusService::show()
{
    viewManager_->open();
}

void DBusService::hide()
{
    viewManager_->close();
}

} // namespace MultitaskView
