/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "Controller.js" as Controller
import IconTheme 1.0
import QtGraphicalEffects 1.0
//import org.ukui.qqc2style.private 1.0 as StylePrivate

Rectangle {
    id : closeButton
    visible : closeButtonVisble
    width : 24
    height : 24
    radius : 14
    color : closeButtonColor

    property var closeButtonController : new Controller.CloseButtonController(multitaskViewModel, closeButton, multitaskViewManager, ["app"])
    property var buttonStateList : ["default", "hover", "click"];
    state : "default"
    property var closeButtonColor : closeButtonDefaultColor
    property string wid : ""
    property alias closeButtonContainMouse: closeButtonMouseArea.containsMouse
    property var imageSource : "qrc:/images/application-exit-symbolic.svg"


    Image {
        id: closeWindow
        anchors.centerIn : parent
        source : imageSource
        sourceSize.width: Math.floor(parent.width)/2
        sourceSize.height: Math.floor(parent.height)/2
    }

    function setStatus(status) {
        if(buttonStateList.indexOf(status) < 0)
        {
            return;
        }
        if (status === state) {
            return
        } else {
            state = status;
        }
    }

    function setDefaultStatus() {

        setStatus("default")
    }

    function setHoverStatus() {

        setStatus("hover")
    }

    function setClickStatus() {

        setStatus("click")
    }

    function setId(wId) {
        wid = wId;
    }

    MouseArea{
        id : closeButtonMouseArea
        anchors.fill : parent
        hoverEnabled : true

        onEntered: {
            closeButtonController.setHoverStatus()
        }

        onExited: {
            closeButtonController.setDefaultStatus()
            if(!previewMouseArea.containsMouse)
            {
                previewWindow.setDefaultStatus()
            }
        }

        onPressed: {
            closeButtonController.setClickStatus()
        }

        onReleased: {
            closeButtonController.setHoverStatus()
        }

        onClicked: {
            controller.removePreviewWindow(wid);
        }
    }
}
