/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
   窗口区域的框架创建，加载创建每一个预览窗口；
   计算每一个预览窗口的位置、大小，以及绘制预览图的内容
*/
import QtQuick 2.9
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import QtQuick.Controls.Styles 1.4
import "Controller.js" as Controller

FocusScope{
    id : wer
    width : 100
    height : 100
    property ListModel appModel : null
    property var rootRect : null
    property var appAreaController  : null
    property var keyAppController : null
    property var currentTabIndex: 0

    Rectangle {
        id : appArea
        width : parent.width
        height : parent.height
        color : "transparent"

        ScrollView{
            id : scrollArea
            width : parent.width
            height : parent.height
            property var flowRows : 0
            property var thumbnailHeight : (rootRect.height / 23 * 5) + 2 + 24 + 8
            property ScrollBar vScrollBar: ScrollBar.vertical
            property ListModel windowPosInfoModel: ListModel{}
            contentWidth : parent.width
            contentHeight : flowRows * ((rootRect.height / 23 * 5) + 2 + 24 + 18) - 10 > parent.height ? flowRows * ((rootRect.height / 23 * 5) + 2 + 24 + 18) - 10 : parent.height
            ScrollBar.horizontal.policy : ScrollBar.AlwaysOff
            ScrollBar.vertical.policy : ScrollBar.AsNeeded
            clip : true

            MouseArea{
                anchors.fill : parent
                onClicked : {
                    appAreaController.closeMultitaskView()
                }
            }

            NumberAnimation {
                id : scrollMoveAnimation
                target : scrollArea.vScrollBar
                property : "position"
                from : scrollArea.vScrollBar.position
                duration : 300
                easing.type : Easing.InOutQuart
            }

            Flow{
                id : flowLayout
                spacing : 10
                width : appArea.width
                flow : Flow.LeftToRight

                move :Transition { //子项增加时或者减少时的过度，也就是子项的visible变化的过度
                    NumberAnimation {
                        properties : "x,y";
                        easing.type : Easing.InOutQuad;
                        duration : 300
                    }
                }

                Repeater{
                    property int cnt : 0
                    id : rep
                    model : appModel
                    activeFocusOnTab: appModel.count !== 0 ? true : false
                    focus : true

                    onActiveFocusChanged: {
                        if (rep.activeFocus) {
                            currentTabIndex = 0
                            if (scrollArea.vScrollBar.position > 0) {
                                scrollMoveAnimation.to = 0
                                scrollMoveAnimation.start()
                            }
                        }
                    }

                    function adjustSize(){
                        // 计算当前布局的方式
                        var cols = 0;
                        var lineWidth = appArea.width;
                        var nowlineWidth = 0
                        var nowflowRows = 0
                        var rowWindowNum = 0                  //窗口区域每行有多少窗口
                        var leftX = 0
                        var rightX = 0
                        scrollArea.windowPosInfoModel.clear()

                        // 调整矩形框的大小
                        for(var i = 0; i < rep.cnt; i++)
                        {
                            var lastline = true
                            for(var j = i; j < rep.cnt; j++)
                            {
                                nowlineWidth = nowlineWidth + itemAt(j).trueWidth + 10
                                if (nowlineWidth < lineWidth) {
                                    itemAt(j).width = itemAt(j).trueWidth
                                    cols = j
                                } else {
                                    lastline = false
                                    nowlineWidth = Math.floor(nowlineWidth - itemAt(j).trueWidth - 20)
                                    break;
                                }
                            }
                            if (lastline) {
                                nowlineWidth = nowlineWidth - 10
                            }
                            itemAt(i).width = itemAt(i).trueWidth + (lineWidth - nowlineWidth)/2
                            nowlineWidth = 0
                            i = cols
                            nowflowRows++
                        }
                        scrollArea.flowRows = scrollArea.flowRows === nowflowRows ? scrollArea.flowRows : nowflowRows

                        for(var m = 0; m < rep.cnt; m++)
                        {
                            var winModel = [];
                            cols = 0
                            for(var n = m; n < rep.cnt; n++)
                            {
                                nowlineWidth = nowlineWidth + itemAt(n).width + 10
                                leftX = nowlineWidth - itemAt(n).trueWidth - 10
                                rightX = nowlineWidth - 10
                                if (nowlineWidth < lineWidth) {
                                    cols = n
                                    winModel.push({index: n, leftX: leftX, rightX: rightX, width: itemAt(n).width})
                                } else {
                                    break;
                                }
                            }
                            nowlineWidth = 0
                            rowWindowNum = cols - m + 1
                            m = cols
                            scrollArea.windowPosInfoModel.append({model : winModel, rowWindowCount : rowWindowNum});
                        }
                    }

                    function keyPageDownSelectWIndow()
                    {
                        var currentRow = 0
                        var nextRowWindowNum = 0
                        var currentRowWindowNum = 0
                        var windowNum = 0
                        //计算当前获取键盘焦点的窗口位于第几行
                        for(var i = 0; i < scrollArea.windowPosInfoModel.count; i++)
                        {
                            windowNum += scrollArea.windowPosInfoModel.get(i).rowWindowCount
                            if (currentTabIndex < windowNum)
                            {
                                if (i === scrollArea.windowPosInfoModel.count - 1)
                                {
                                    return currentTabIndex
                                }
                                currentRow = i
                                currentRowWindowNum = scrollArea.windowPosInfoModel.get(i).rowWindowCount
                                nextRowWindowNum = scrollArea.windowPosInfoModel.get(i + 1).rowWindowCount
                                break
                            }
                        }

                        var nextRowModel = scrollArea.windowPosInfoModel.get(currentRow + 1).model
                        var resultIndex = nextRowModel.get(nextRowWindowNum - 1).index
                        var currentNextY = rep.itemAt(resultIndex).mapToItem(scrollArea,0,0).y
                        if (currentNextY + scrollArea.thumbnailHeight > scrollArea.height) {
                            scrollMoveAnimation.to = scrollArea.vScrollBar.position + (currentNextY + scrollArea.thumbnailHeight - scrollArea.height) / scrollArea.contentHeight
                            scrollMoveAnimation.start()
                        }
                        return resultIndex
                    }

                    function keyDownSelectWIndow()
                    {
                        var currentRow = 0
                        var nextRowWindowNum = 0
                        var currentRowWindowNum = 0
                        var windowNum = 0
                        //计算当前获取键盘焦点的窗口位于第几行
                        for(var i = 0; i < scrollArea.windowPosInfoModel.count; i++)
                        {
                            windowNum += scrollArea.windowPosInfoModel.get(i).rowWindowCount
                            if (currentTabIndex < windowNum)
                            {
                                if (i === scrollArea.windowPosInfoModel.count - 1)
                                {
                                    return currentTabIndex
                                }
                                currentRow = i
                                currentRowWindowNum = scrollArea.windowPosInfoModel.get(i).rowWindowCount
                                nextRowWindowNum = scrollArea.windowPosInfoModel.get(i + 1).rowWindowCount
                                break
                            }
                        }

                        //计算当前获取键盘焦点窗口的leftX rightX
                        var currentRowModel = scrollArea.windowPosInfoModel.get(currentRow).model
                        var currentIndexLeftX = 0
                        var currentIndexRightX = 0
                        for (var i = 0; i < currentRowWindowNum; i++)
                        {
                            if (currentTabIndex === currentRowModel.get(i).index)
                            {
                                currentIndexLeftX = currentRowModel.get(i).leftX
                                currentIndexRightX = currentRowModel.get(i).rightX
                                break
                            }
                        }

                        //判断应该跳转到下一行的哪个窗口
                        var nextRowModel = scrollArea.windowPosInfoModel.get(currentRow + 1).model
                        var resultIndex = 0
                        var result = 0
                        for (var i = 0; i < nextRowWindowNum; i++)
                        {
                            var temp = 0
                            if ((nextRowModel.get(i).leftX > currentIndexLeftX && nextRowModel.get(i).rightX > currentIndexRightX) &&
                                    i === 0)
                            {
                                resultIndex = nextRowModel.get(i).index
                                return resultIndex
                            }
                            else if ((nextRowModel.get(i).leftX < currentIndexLeftX && nextRowModel.get(i).rightX < currentIndexRightX) &&
                                     i === nextRowWindowNum - 1)
                            {
                                resultIndex = nextRowModel.get(i).index
                                return resultIndex
                            }
                            else
                            {
                                if (nextRowModel.get(i).leftX < currentIndexLeftX && nextRowModel.get(i).rightX > currentIndexLeftX)
                                {
                                    temp = nextRowModel.get(i).rightX - currentIndexLeftX
                                }
                                else if (nextRowModel.get(i).leftX > currentIndexLeftX && nextRowModel.get(i).rightX < currentIndexRightX)
                                {
                                    temp = nextRowModel.get(i).width
                                }
                                else if (nextRowModel.get(i).leftX < currentIndexRightX && nextRowModel.get(i).rightX > currentIndexRightX)
                                {
                                    temp = currentIndexRightX - nextRowModel.get(i).leftX
                                }
                                else if (nextRowModel.get(i).leftX <= currentIndexLeftX && nextRowModel.get(i).rightX >= currentIndexRightX)
                                {
                                    temp = nextRowModel.get(i).width
                                }
                                else
                                {
                                    continue
                                }

                                if (result < temp)
                                {
                                    resultIndex = nextRowModel.get(i).index
                                    result = temp
                                }
                            }
                        }
                        return resultIndex
                    }

                    function keyPageUpSelectWIndow()
                    {
                        var currentRow = 0
                        var beforeRowWindowNum = 0
                        var currentRowWindowNum = 0
                        var windowNum = 0

                        //计算当前获取键盘焦点的窗口位于第几行
                        for(var i = 0; i < scrollArea.windowPosInfoModel.count; i++)
                        {
                            windowNum += scrollArea.windowPosInfoModel.get(i).rowWindowCount
                            if (currentTabIndex < windowNum)
                            {
                                if (i === 0)
                                {
                                    return currentTabIndex
                                }
                                currentRow = i
                                currentRowWindowNum = scrollArea.windowPosInfoModel.get(i).rowWindowCount
                                beforeRowWindowNum = scrollArea.windowPosInfoModel.get(i - 1).rowWindowCount
                                break
                            }
                        }

                        //判断应该跳转到上一行的哪个窗口
                        var beforeRowModel = scrollArea.windowPosInfoModel.get(currentRow - 1).model
                        var resultIndex = beforeRowModel.get(0).index
                        var currentNextY = rep.itemAt(resultIndex).mapToItem(scrollArea,0,0).y
                        if (currentNextY < 0 ) {
                            scrollMoveAnimation.to = scrollArea.vScrollBar.position - (-currentNextY) / scrollArea.contentHeight
                            scrollMoveAnimation.start()
                        }
                        return resultIndex
                    }


                    function keyUpSelectWIndow()
                    {
                        var currentRow = 0
                        var beforeRowWindowNum = 0
                        var currentRowWindowNum = 0
                        var windowNum = 0

                        //计算当前获取键盘焦点的窗口位于第几行
                        for(var i = 0; i < scrollArea.windowPosInfoModel.count; i++)
                        {
                            windowNum += scrollArea.windowPosInfoModel.get(i).rowWindowCount
                            if (currentTabIndex < windowNum)
                            {
                                if (i === 0)
                                {
                                    return currentTabIndex
                                }
                                currentRow = i
                                currentRowWindowNum = scrollArea.windowPosInfoModel.get(i).rowWindowCount
                                beforeRowWindowNum = scrollArea.windowPosInfoModel.get(i - 1).rowWindowCount
                                break
                            }
                        }

                        //计算当前获取键盘焦点窗口的leftX rightX
                        var currentRowModel = scrollArea.windowPosInfoModel.get(currentRow).model
                        var currentIndexLeftX = 0
                        var currentIndexRightX = 0
                        for (var i = 0; i < currentRowWindowNum; i++)
                        {
                            if (currentTabIndex === currentRowModel.get(i).index)
                            {
                                currentIndexLeftX = currentRowModel.get(i).leftX
                                currentIndexRightX = currentRowModel.get(i).rightX

                                break
                            }
                        }

                        //判断应该跳转到上一行的哪个窗口
                        var beforeRowModel = scrollArea.windowPosInfoModel.get(currentRow - 1).model
                        var resultIndex = 0
                        var result = 0
                        for (var i = 0; i < beforeRowWindowNum; i++)
                        {
                            var temp = 0
                            if (beforeRowModel.get(i).leftX < currentIndexLeftX && beforeRowModel.get(i).rightX > currentIndexLeftX)
                            {
                                temp = beforeRowModel.get(i).rightX - currentIndexLeftX
                            }
                            else if (beforeRowModel.get(i).leftX > currentIndexLeftX && beforeRowModel.get(i).rightX < currentIndexRightX)
                            {
                                temp = beforeRowModel.get(i).width
                            }
                            else if (beforeRowModel.get(i).leftX < currentIndexRightX && beforeRowModel.get(i).rightX > currentIndexRightX)
                            {
                                temp = currentIndexRightX - beforeRowModel.get(i).leftX
                            }
                            else if (beforeRowModel.get(i).leftX < currentIndexLeftX && beforeRowModel.get(i).rightX > currentIndexRightX)
                            {
                                temp = beforeRowModel.get(i).width
                            }
                            else
                            {
                                continue
                            }

                            if (result < temp)
                            {
                                resultIndex = beforeRowModel.get(i).index
                                result = temp
                            }
                        }
                        return resultIndex
                    }

                    onItemAdded : {
                        cnt++;
                        adjustSize();
                    }

                    onItemRemoved : {
                        cnt--;
                        adjustSize();
                    }

                    Keys.onPressed: {
                        if (event.key === Qt.Key_Left)
                        {
                            if (currentTabIndex > 0)
                            {
                                currentTabIndex = currentTabIndex - 1
                                var currentNextY = rep.itemAt(currentTabIndex).mapToItem(scrollArea,0,0).y
                                if (currentNextY < 0 ) {
                                    scrollMoveAnimation.to = scrollArea.vScrollBar.position - (-currentNextY) / scrollArea.contentHeight
                                    scrollMoveAnimation.start()
                                }
                            }
                            else if(currentTabIndex === 0)
                            {
                                currentTabIndex = appModel.count - 1
                                var currentNextY = rep.itemAt(currentTabIndex).mapToItem(scrollArea,0,0).y
                                if (currentNextY + scrollArea.thumbnailHeight > scrollArea.height) {
                                    scrollMoveAnimation.to = scrollArea.vScrollBar.position + (currentNextY + scrollArea.thumbnailHeight - scrollArea.height) / scrollArea.contentHeight
                                    scrollMoveAnimation.start()
                                }
                            }
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_Right )
                        {
                            if (currentTabIndex < appModel.count - 1)
                            {
                                var currentBeforeY = rep.itemAt(currentTabIndex).mapToItem(scrollArea,0,0).y
                                currentTabIndex = currentTabIndex + 1
                                var currentNextY = rep.itemAt(currentTabIndex).mapToItem(scrollArea,0,0).y

                                if(currentBeforeY < currentNextY) {
                                    if (currentNextY + scrollArea.thumbnailHeight > scrollArea.height) {
                                        scrollMoveAnimation.to = scrollArea.vScrollBar.position + (currentNextY + scrollArea.thumbnailHeight - scrollArea.height) / scrollArea.contentHeight
                                        scrollMoveAnimation.start()
                                    }
                                }
                            }
                            else if(currentTabIndex === appModel.count - 1)
                            {
                                currentTabIndex = 0
                                if (scrollArea.vScrollBar.position > 0) {
                                    scrollMoveAnimation.to = 0
                                    scrollMoveAnimation.start()
                                }
                            }

                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_Up)
                        {
                            if (scrollArea.flowRows === 1)
                            {
                                return
                            }
                            currentTabIndex = keyUpSelectWIndow()
                        }
                        else if (event.key === Qt.Key_Down)
                        {
                            if (scrollArea.flowRows === 1)
                            {
                                return
                            }
                            currentTabIndex = keyDownSelectWIndow()
                        }
                        else if (event.key === Qt.Key_Tab)
                        {
                            if (event.modifiers & Qt.MetaModifier)
                            {
                                allcontroller.closeMultitaskView()
                                return
                            }

                            currentTabIndex = 0
                            event.accepted = false
                        }
                        else if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter || event.key === Qt.Key_Space)
                        {
                            keyAppController.onWindowClicked(appModel.get(currentTabIndex).id);
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_End)
                        {
                            currentTabIndex = appModel.count - 1
                            var currentNextY = rep.itemAt(currentTabIndex).mapToItem(scrollArea,0,0).y
                            if (currentNextY + scrollArea.thumbnailHeight > scrollArea.height) {
                                scrollMoveAnimation.to = scrollArea.vScrollBar.position + (currentNextY + scrollArea.thumbnailHeight - scrollArea.height) / scrollArea.contentHeight
                                scrollMoveAnimation.start()
                            }
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_Home)
                        {
                            currentTabIndex = 0
                            if (scrollArea.vScrollBar.position > 0) {
                                scrollMoveAnimation.to = 0
                                scrollMoveAnimation.start()
                            }
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_PageUp)
                        {
                            currentTabIndex = keyPageUpSelectWIndow()
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_PageDown)
                        {
                            currentTabIndex = keyPageDownSelectWIndow()
                            event.accepted = true
                        }
                        else if (event.key === Qt.Key_Escape)
                        {
                            allcontroller.closeMultitaskView()
                        }
                    }


                    Rectangle{
                        id : thumbnailitem
                        width: thumbnailitem.thumbwidth() + 12 > appArea.width / 10 ? thumbnailitem.thumbwidth() + 12  : appArea.width / 10
                        height : (rootRect.height / 23 * 5) + 2 + 24 + 8
                        color : "transparent"
                        radius : 14
                        focus: true
                        property var trueWidth : thumbnailitem.thumbwidth() + 12 > appArea.width / 10 ? thumbnailitem.thumbwidth() + 12  : appArea.width / 10

                        states: [
                            State {
                                name: "active"
                                when: rep.activeFocus && index === currentTabIndex
                                PropertyChanges { target: appWindow; tabBorderWidth :4 }
                            },
                            State {
                                name: "thumbnailitemactive"
                                when: thumbnailitem.activeFocus
                                PropertyChanges { target: appWindow; tabBorderWidth :4 }
                            }
                        ]

                        function thumbwidth(){
                            var max_height = rootRect.height / 23 * 5 - 12
                            var max_width = appArea.width / 3 - 12
                            var oriW = model.width;
                            var oriH = model.height;
                            var dh, dw;
                            var isfind = 0;
                            // 情况2
                            dh = max_height
                            dw = oriW * dh / oriH;
                            if(dw <= max_width){
                                isfind = 1;
                            }
                            // 情况1
                            if(!isfind){
                                dw = max_width
                                dh = oriH * max_width / oriW
                            }
                            return dw
                        }

                        function thumbheight(){
                            var max_height = rootRect.height / 23 * 5 - 12
                            var max_width = appArea.width / 3 - 12
                            var oriW = model.width;
                            var oriH = model.height;
                            var dh, dw;

                            var isfind = 0;
                            // 情况2
                            dh = max_height
                            dw = oriW * dh / oriH
                            if(dw <= max_width){
                                isfind = 1;
                            }
                            // 情况1
                            if(!isfind){
                                dw = max_width
                                dh = oriH * max_width / oriW
                            }
                            return dh
                        }

                        Rectangle {
                            id : appWindowAnchor
                            anchors.verticalCenter: parent.verticalCenter
                            x:thumbnailitem.width-appWindowAnchor.width
                            width : thumbnailitem.thumbwidth() + 12 > appArea.width / 10 ? thumbnailitem.thumbwidth() + 12  : appArea.width / 10
                            height : parent.height-8
                            color : "transparent"

                            MasterPreviewWindow {
                                id : appWindow
                                width : thumbnailitem.thumbwidth() + 12 > appArea.width / 10 ? thumbnailitem.thumbwidth() + 12  : appArea.width / 10
                                height : (rootRect.height/23*5) + 2 + 24
                                previewAreaWidth : thumbnailitem.thumbwidth() + 12 > appArea.width / 10 ? thumbnailitem.thumbwidth() + 12  : appArea.width / 10
                                previewAreaHeight :  rootRect.height / 23 * 5
                                windowType : "app"
                                controller : new Controller.AppController(multitaskViewModel, appWindow, multitaskViewManager, ["none"]);

                                onWinowDragActiveChanged : {
                                    if(!appWindow.winowDragActive) {
                                        appWindow.zoomOut()
                                        var fromPos  = appWindow.mapToItem(appWindowAnchor,0,0)

                                        appWindow.posX = fromPos.x
                                        appWindow.posY = fromPos.y
                                        appAreaController.onSourceDropped(appWindow,rootWindow.highlightDesktop,rootWindow.highlightDesktop)
                                    } else {
                                        appWindow.zoomIn()
                                    }
                                }

                                states :  [
                                    State {
                                        name : "default"
                                        PropertyChanges { target : appWindow; borderWidth : 0 }
                                        PropertyChanges { target : appWindow; closeButtonVisble : false }
                                    },

                                    State {
                                        name : "hover"
                                        PropertyChanges { target : appWindow; borderWidth : 4 }
                                        PropertyChanges { target : appWindow; closeButtonVisble : true }
                                    }
                                ]

                                Rectangle {
                                    id : appWindowBack
                                    anchors.horizontalCenter : appWindow.horizontalCenter
                                    y : 2 + 24 + 6
                                    height : appWindow.previewAreaHeight-12
                                    width : appWindow.previewAreaWidth-12
                                    color : "transparent"
                                    clip : true
                                    visible : true
                                    layer.enabled : multitaskViewManager.useOpenGL() ? true : false
                                    layer.effect : OpacityMask{
                                        maskSource : Rectangle{
                                            height : appWindow.previewAreaHeight-12
                                            width : appWindow.previewAreaWidth-12
                                            radius : 8
                                        }
                                    }

                                    KWinComponents.WindowThumbnailItem{
                                        id : tmp
                                        anchors.centerIn : parent
                                        z: appWindow.onTop ? 100 : 20
                                        height : thumbnailitem.thumbheight()
                                        width : thumbnailitem.thumbwidth()
                                        wId : model.uuid
                                        clip : true
                                    }
                                }

                                Component.onCompleted : {
                                    appWindow.setCaption(model.caption)
                                    appWindow.setId(model.uuid)
                                }
                            }
                        }
                    }//Rectangle
                }//Repeater
            }//Flow

        }//ScrollView

    }

}
