/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller

Item {
    id : newDesktopButton
    width : desktopThumbnailWidth + 2 * desktopBorderWidth
    height : desktopThumbnailHeight + desktopCaptionHeight + 2 * desktopBorderWidth +  24
    visible : desktopListView.count < 16 ? true : false

    focus: true
    activeFocusOnTab : true

    onActiveFocusChanged: {
        if (newDesktopButton.activeFocus) {
            desktopListView.currentIndex = desktopModel.count - 1
            if (desktopListView.contentY < desktopListView.contentHeight-desktopListView.height) {
                contentMoveAnimation.to = desktopListView.contentHeight-desktopListView.height
                contentMoveAnimation.start()
            }
        }
    }

    Keys.onPressed: {
        if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter || event.key === Qt.Key_Space)
        {
            keyNewDesktopController.onWindowClicked()
            event.accepted = true
        }
        else if (event.key === Qt.Key_Escape)
        {
            allcontroller.closeMultitaskView()
        }
        else if (event.modifiers & Qt.MetaModifier)
        {
            if (event.key === Qt.Key_Tab)
            {
                allcontroller.closeMultitaskView()
            }
        }
    }

    states: State {
        name: "active"; when: newDesktopButton.activeFocus
        PropertyChanges { target: spacingRect; tabBorderWidth : 4}
    }

    MasterPreviewWindow {
        id : spacingRect
        width : desktopThumbnailWidth + 2 * desktopBorderWidth
        height : desktopThumbnailHeight + desktopCaptionHeight + 2 * desktopBorderWidth
        previewAreaHeight : desktopThumbnailHeight+ 2 * desktopBorderWidth
        previewAreaWidth :desktopThumbnailWidth+ 2 * desktopBorderWidth
        captionColor: rootWindow.desktopCaptionColor
        iconVisble : false
        windowType : "creation"
        controller : new Controller.CreationController(multitaskViewModel, spacingRect, multitaskViewManager, ["app"]);

        DropArea {
            id : newDesktopDropArea
            anchors.fill : parent

            onEntered : {
                createDesktopButton.color = rootWindow.createDesktopButtonReceiveColor
            }

            onExited : {
                createDesktopButton.color = rootWindow.createDesktopButtonDefaultColor
            }

            onDropped : {
                spacingRect.controller.onSourceDropped(drop.source, rootWindow.highlightDesktop,desktopModel.count);
                rootWindow.updateDesktopWindowModel()
            }
        }

        Rectangle {
            id : createDesktopButton
            width : desktopThumbnailWidth
            height : desktopThumbnailHeight
            x : desktopBorderWidth
            y : desktopCaptionHeight+desktopBorderWidth
            radius : 8
            color : rootWindow.createDesktopButtonDefaultColor

            Rectangle {
                anchors.centerIn : parent
                color : rootWindow.desktopCaptionColor
                width : newDesktopButton.height / 4
                height : newDesktopButton.height / 100
            }

            Rectangle {
                anchors.centerIn : parent
                color : rootWindow.desktopCaptionColor
                width : newDesktopButton.height / 100
                height : newDesktopButton.height / 4
            }
        }

        Component.onCompleted : {
            spacingRect.setCaption(qsTr("New Desktop"))
            spacingRect.setId(0)
        }
    }
}
