/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
    此文件是虚拟桌面的预览窗口的内容设置
*/

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller
Rectangle {
    id : desktopListViewDelegate
    width : desktopThumbnailWidth + 2 * desktopBorderWidth
    height : desktopThumbnailHeight + desktopCaptionHeight + 2 * desktopBorderWidth + 2
    color : "transparent"
    activeFocusOnTab : index === 0 ? true : false
    focus: true

    states: State {
        name: "active"; when: desktopListViewDelegate.activeFocus
        PropertyChanges { target: desktopDragRect; tabBorderWidth : 4 }
    }

    Keys.onPressed: {
        if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter || event.key === Qt.Key_Space)
        {
            keyDesktopController.setCurrentDesktop(value)
            keyDesktopController.onWindowClicked(value)
            event.accepted = true
        }
        else if (event.key === Qt.Key_Escape)
        {
            allcontroller.closeMultitaskView()
        }
        else if (event.modifiers & Qt.MetaModifier)
        {
            if (event.key === Qt.Key_Tab)
            {
                allcontroller.closeMultitaskView()
            }
        }
    }

    onActiveFocusChanged: {
        if (index === 0 )
        {
            if (desktopListViewDelegate.activeFocus) {
                desktopListView.currentIndex = 0
                if (desktopListView.contentY > 0){
                    contentMoveAnimation.to = 0
                    contentMoveAnimation.start()
                }
            }
        }
    }

    MasterPreviewWindow{
        id : desktopDragRect
        width : desktopListViewDelegate.width
        height : desktopListViewDelegate.height
        dragAxis : Drag.YAxis
        previewAreaWidth : desktopThumbnailWidth + 2 * desktopBorderWidth
        previewAreaHeight : desktopThumbnailHeight + 2 * desktopBorderWidth
        iconVisble : false
        opacity : 1
        wid: value //工作区id会根据工作区的删除交换等操作改变，因此不使用函数赋值
        windowType : "desktop"
        controller : new Controller.DesktopController(multitaskViewModel, desktopDragRect, multitaskViewManager, ["app","desktop"]);
        captionText : qsTr("Desktop") + " " + value
        captionColor: rootWindow.desktopCaptionColor
        property var nowDragActive : false

        onWinowDragActiveChanged : {
            if(winowDragActive)
            {
                desktopMaskColor.visible = false
                opacity = 0.75
            }
            else
            {
                opacity = 1
            }

            if (winowDragActive !== nowDragActive) {
                nowDragActive = winowDragActive
            }
        }

        onNowDragActiveChanged : {
            if(!desktopDragRect.winowDragActive) {
                desktopDragRect.posX = 0
                desktopDragRect.posY = 0
                desktopAreaController.onSourceDropped(desktopDragRect,rootWindow.highlightDesktop,rootWindow.highlightDesktop)
            }
        }

        states :  [
            State {
                name : "default"
                PropertyChanges { target : desktopDragRect; borderWidth : rootWindow.highlightDesktop == value ? 4  : 0 }
                PropertyChanges { target : desktopDragRect; closeButtonVisble : false }
            },

            State {
                name : "hover"
                PropertyChanges { target : desktopDragRect; borderWidth : 4 }
                PropertyChanges { target : desktopDragRect; closeButtonVisble : true }
            }
        ]

        Rectangle {
            id: workspace
            anchors.horizontalCenter : desktopDragRect.horizontalCenter
            x : desktopBorderWidth
            y : desktopCaptionHeight+desktopBorderWidth+2
            width : desktopThumbnailWidth
            height : desktopThumbnailHeight
            color : "black"
            clip :true
            visible : true
            layer.enabled : multitaskViewManager.useOpenGL() ? true : false
            layer.effect : OpacityMask{
                maskSource : Rectangle{
                    width : desktopThumbnailWidth
                    height : desktopThumbnailHeight
                    radius : 8
                }
            }

            KWinComponents.DesktopBackground {
                id : desktopThumbnailBackground
                anchors.centerIn : parent
                width : desktopThumbnailWidth
                height : desktopThumbnailHeight
                blurAlpha : 0
            }
            DropArea {
                id : previewDropArea
                anchors.fill : parent
                visible: !desktopDragRect.winowDragActive

                onEntered : {
                    desktopMaskColor.visible = true
                    desktopDragRect.controller.onSourceEntered(drag.source,drag.source.wid,desktopDragRect.wid);
                }

                onExited : {
                    desktopMaskColor.visible = false
                }

                onDropped : {
                    desktopMaskColor.visible = false
                    desktopDragRect.controller.onSourceDropped(drop.source, rootWindow.highlightDesktop,desktopDragRect.wid);
                    rootWindow.updateDesktopWindowModel()
                }
            }

            Repeater {
                id : desktopWindowRepeater
                model : desktopModel.get(index).model

                KWinComponents.WindowThumbnailItem{
                    id : desktopWindowThumbnail
                    width : model.width * desktopScale
                    height : model.height * desktopScale
                    visible : !(model.minimized)
                    x : (model.x - rootRect.x) * desktopScale
                    y : (model.y - rootRect.y) * desktopScale
                    wId : model.id
                    clip : true
                }
            }

            Rectangle {
                id : desktopMaskColor
                visible : false
                anchors.fill: parent
                color :  rootWindow.desktopMaskReceiveColor
            }
        }
    }
}
