/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// Controller 是基类，用于各种根窗口或者非应用程序预览窗口的窗口, 实现了设计文档中预览窗口类中的大部分函数的转发接口；
// TabletAppController 是应用程序预览窗口控制器类，实现了点击窗口时激活的接口和删除应用程序预览窗口的接口。
// ClearAllController 是全部清除按钮控制器类，实现了清除所有窗口的接口和设置按钮按下状态的接口。
// 控制器基类
class Controller {
    constructor(model, view, manager, acceptedTypeSet) {
        this.model = model;
        this.view = view;
        this.manager = manager;
        this.acceptedTypeSet = acceptedTypeSet;
    }

    initViewAnimation() {
        console.log("控制器模块开始判断是否有窗口")
        var noAppWindow = "true"
        for (var i = 0; i < this.model.logicalScreenCount; i++)
        {
            if (this.model.getAppWindowList(this.model.getCurrentDesktop(), i).length === 0)
            {
                continue;
            }
            else
            {
                noAppWindow = "false"
            }
        }

        console.log("noAppWindow is",noAppWindow)
        if (noAppWindow === "true")
        {
            // no app window
            // calling no window animation
            this.view.startNoWindowTipsAnimation();
            // view will call disappear animation itself.
            // view will call close function then.
        }
        else
        {
            // call AllClear Button animation
            this.view.startSequentialLoadWindowTimer();
        }
    }

    //点击空白处关闭多任务视图
    viewSpacingClickedController()
    {
        if (this.model.isTabletMode) {
            this.manager.showDesktop(true);
        }
        this.view.viewSpacingClickedController()
    }

    // 关闭多任务视图
    closeMultitaskView() {
        this.manager.close();
    }

    // 窗口点击事件处理
    onWindowClicked(windowId) {
    }

    startClearAllBtnDisappearAni()
    {
        this.view.startClearAllBtnDisappearAni();
    }

    // 删除预览窗口
    removePreviewWindow(windowId) {}

    // 设置默认态
    setDefaultStatus() {
        this.view.setDefaultStatus();
    }

    // 设置 Hover 态
    setHoverStatus() {
        this.view.setHoverStatus();
    }

    //设置长按状态
    setPressStatus()
    {
        this.view.setHoverStatus();
    }

    // 设置拖拽态
    setDragStatus() {
        this.view.setDragStatus();
    }

    // 设置拖拽接收态
    setReceiveStatus() {
        this.view.setReceiveStatus();
    }

    startSpaceClickedPreWindowAnimation()
    {
        this.view.startSpaceClickedPreWindowAnimation();
    }
}

class TabletAppController extends Controller {
    constructor(model, view, manager, acceptedTypeSet) {
        super(model, view, manager, acceptedTypeSet);
    }

    //删除当前桌面当前屏幕下的所有窗口
    removeAllAppWindows( screenIndex)
    {
        this.model.removeAllAppWindowOnDesktop(this.model.getCurrentDesktop(), screenIndex)
    }

    // 窗口点击事件处理
    onWindowClicked(windowId) {
        // activate windowId
        this.model.activateAppWindow(windowId);
        if (this.model.isTabletMode)
        {
            this.view.startActiveWindowAnimation();
            // view will call close function
        }
    }

    // 向上滑动确认关闭
    slideUpCloseWindow() {
        this.view.startCloseWindowAnimation();
    }

    // 向上滑动取消关闭
    startResumeWindowAnimation() {
        this.view.startResumeWindowAnimation();
    }

    // 删除预览窗口
    removePreviewWindow(windowId) {
        if (this.model.hasModalChildWindow(windowId))
        {
            this.onWindowClicked(windowId);
            // If window has modal child window,
            // controller calls close force.
            this.closeMultitaskView();
        }
        else
        {
            // delete app
            this.model.removeAppWindow(windowId);
        }
    }

    startClearAllWindowAnimation(/*unused*/) {
        this.view.startClearAllWindowAnimation();
    }
}

class CloseButtonController extends Controller {
    constructor(model, view, manager, acceptedTypeSet) {
        super(model, view, manager, acceptedTypeSet);
    }
}

class ClearAllController extends Controller {
    constructor(model, view, manager, acceptedTypeSet) {
        super(model, view, manager, acceptedTypeSet);
    }

    // 窗口点击事件处理
    onWindowClicked(/*unused*/) {
        this.view.startClearAllWindowAnimation();
    }

    // 窗口按下事件处理
    setPressStatus() {
        this.view.setPressStatus();
    }
}
