/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import IconTheme 1.0
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0

Rectangle {
    id : tabletPreviewWindow
    width : 498
    height : 320
    color : "transparent"

    property ListModel premodel : null
    property bool closeButtonVisble : false
    property real borderWidth : 0
    property var screenIndex : null
    property var stateList : ["default", "hover"];
    state : "default"
    property string wid : ""
    //当前预览窗口在桌面上的未缩放前的实际宽度
    property int preViewNoScaledWidth : -1
    //当前预览窗口在桌面上的未缩放前的实际高度
    property int preViewNoScaledHeight : -1
    //当前预览窗口在桌面上的未缩放前的X坐标
    property int preViewNoScaledXCoordinate : -1
    //当前预览窗口在桌面上的未缩放前的Y坐标
    property int preViewNoScaledYCoordinate : -1
    property var captionText  : ""
    property var appController : new TabletController.TabletAppController(multitaskViewModel, appWindow, multitaskViewManager, ["app"])
    property var thumbAnchorRootMapGloableX : mapToGlobal(thumbAnchorRoot.x,thumbAnchorRoot.y).x
    property var thumbAnchorRootMapGloableY : mapToGlobal(thumbAnchorRoot.x,thumbAnchorRoot.y).y
    property int thumbnailRadius: multitaskViewManager.useOpenGL() ? 14 : 0
    property var currentDesktopWindowNumber : 0

    /***************************************动画***************************/
    //全部清除窗口的动效
    ParallelAnimation
    {
        id:clearAllWindowAnimation
        NumberAnimation {
            target: tabletPreviewWindow
            properties: "y"
            to: Math.floor(tabletPreviewWindow.y - tabletPreviewWindow.height * 2)
            duration: 300
            easing.type: Easing.InOutQuart
        }

        NumberAnimation {
            target: tabletPreviewWindow
            properties: "opacity"
            to: 0
            duration: 300
            easing.type: Easing.InOutQuart
        }

        onFinished: {
            getOtherModelNumber()
            if (premodel.count <= 1)
            {
                appController.removeAllAppWindows(screenIndex)
                if (currentDesktopWindowNumber <= 0)
                {
                    console.log("clearAllWindowAnimation")
                    qmlViewClosed()
                }
            }
            tabletPreviewWindow.removeAppWindow(wid)
        }
    }

    //点击空白处关闭多任务视图动效（小窗口逐渐在原地消失）
    NumberAnimation {
        id : spaceClickedPreWindowAnimation
        target : tabletPreviewWindow
        property : "opacity"
        duration : 300
        easing.type : Easing.InOutQuart
        from : 1
        to : 0
        onFinished: {
            console.log("spaceClickedPreWindowAnimation")
            qmlViewClosed()
        }
    }

    //! 上滑关闭窗口动效
    SmoothedAnimation {
        id : slideUpToCloseWindowAnimation
        target : tabletPreviewWindowMove
        property : "y"
        //因为需要上滑完全离开屏幕 尤其第二行预览窗口，故需要-（预览窗口距离屏幕顶端的距离+预览窗口本身高度）
        to : Math.floor(0 - appWindowBack.y- tabletPreviewWindowMove.height)
        velocity : 1100
        reversingMode:  SmoothedAnimation.Eased
        onFinished: {
            removePreviewWindow(wid)
        }
    }
    //上滑窗口 放大窗口
    NumberAnimation {
        id : enlargeWindow
        target: tabletPreviewWindowMove
        property : "scale"
        duration : 200
        to : 1.05
    }

    //取消上滑后，恢复到原始位置的动画
    ParallelAnimation {
        id : appWindowReturnBackAnimation
        SpringAnimation {
            target : tabletPreviewWindow
            property : "y"
            to : Math.floor(tabletPreviewWindowMove.y)
            easing.type : Easing.InOutQuart
            damping : 0.15                         // 弹簧阻尼值。默认值为0，取值0~1.0。控制
            spring : 2                          // 弹性。有效值范围是0~5.0，默认值为0(禁用了spring的动作)。
        }

        NumberAnimation {
            target: tabletPreviewWindowMove
            property : "scale"
            duration : 200
            to : 1
        }
    }

    //点击应用程序预览窗口激活应用程序时的动画特效
    ParallelAnimation{
        id : activateWindowAnimation
        NumberAnimation{
            target : thumbAnchorRoot
            property : "x"
            from : Math.floor(thumbAnchorRootMapGloableX)
            to : preViewNoScaledXCoordinate
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "y"
            from : Math.floor(thumbAnchorRootMapGloableY)
            to : preViewNoScaledYCoordinate
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "width"
            to : preViewNoScaledWidth
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "height"
            to :  preViewNoScaledHeight
            duration : 250
            easing.type : Easing.InOutQuart
        }
        onFinished: {
            console.log("activateWindowAnimation")
            qmlViewClosed()
        }
    }


    /***************************************函数*************************/
    function getOtherModelNumber()
    {
        currentDesktopWindowNumber = 0
        var screencount = screencnt
        for (var i = 0; i < screencount; i++)
        {
            if (i !== screenIndex)
            {
                var list  = allrootRepeater.itemAt(i).tabletViewRootModel
                currentDesktopWindowNumber += list.count
            }
        }
    }

    function removePreviewWindow(wid)
    {
        appController.removePreviewWindow(wid)
        getOtherModelNumber()
        if (premodel.count <= 1 && currentDesktopWindowNumber <= 0)
        {
            console.log("removePreviewWindow")
            qmlViewClosed()
        }
        tabletPreviewWindow.removeAppWindow(wid)
    }

    function removeAppWindow(wid){
        for(var i = 0; i < premodel.count; i++){
            var info = premodel.get(i);
            if(info.uuid === wid) {
                premodel.remove(i);
                return;
            }
        }
    }

    function appWindowClicked(){
        if (!multitaskViewModel.isTabletMode)
        {
            appController.onWindowClicked(wid);
            console.log("appWindowClicked")
            qmlViewClosed()
        }
        else
        {
            thumbAnchorRootMapGloableY = Math.floor(mapToGlobal(thumbMove.x,thumbMove.y).y)
            thumbAnchorRootMapGloableX = Math.floor(mapToGlobal(thumbMove.x,thumbMove.y).x)
            thumbAnchorRoot.thumbClicked = true
            captionRow.visible = false
            thumbBorder.border.width  = 0
            preWindowCloseButton.visible = false
            appController.onWindowClicked(wid);
        }
    }

    function setStatus(status) {
        if(stateList.indexOf(status) < 0)
        {
            return;
        }
        if (status === state) {
            return
        }
        state = status;
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function startClearAllWindowAnimation()
    {
        clearAllWindowAnimation.start()
    }

    function startSpaceClickedPreWindowAnimation()
    {
        spaceClickedPreWindowAnimation.start()
    }

    function startCloseWindowAnimation()
    {
        slideUpToCloseWindowAnimation.start()
    }

    function startResumeWindowAnimation()
    {
        appWindowReturnBackAnimation.start()
    }

    function startActiveWindowAnimation()
    {
        activateWindowAnimation.start()
    }

    function setCaption(titleText) {
        caption.text = titleText;
    }

    function setId(wId) {
        wid = wId;
    }

    function setPreViewNoScaledWidth(Width) {
        preViewNoScaledWidth = Width;
    }

    function setPreViewNoScaledHeight(Height) {
        preViewNoScaledHeight = Height;
    }

    function setPreViewNoScaledXCoordinate(X) {
        preViewNoScaledXCoordinate = X;
    }

    function setPreViewNoScaledYCoordinate(Y) {
        preViewNoScaledYCoordinate = Y;
    }

    //控制白色边框是否显示
    MouseArea{
        id : preWindowMouseArea
        anchors.fill : parent
        hoverEnabled : true

        onEntered : {
            appController.setHoverStatus();
        }

        onExited: {
            if(!preWindowCloseButton.closeButtonContainMouse)
            {
                appController.setDefaultStatus();
            }
        }

        onClicked: {
            appWindowClicked()
        }

        Rectangle{
            id : tabletPreviewWindowMove
            width : Math.floor(tabletPreviewWindow.width)
            height : Math.floor(tabletPreviewWindow.height)
            color : "transparent"
            radius : tabletPreviewWindow.thumbnailRadius

            ColumnLayout {
                id : columnLayout
                spacing : iconToThumbSpacing
                anchors.centerIn : parent
                Layout.fillHeight : true
                width : Math.floor(tabletPreviewWindow.width)

                RowLayout {
                    id : captionRow
                    spacing : 12 * tabletScaleFactor
                    Layout.fillHeight : true

                    Rectangle {
                        id : iconItem
                        height : Math.floor(iconAndCloseButtonHeight)
                        width : Math.floor(iconAndCloseButtonHeight)
                        color : "transparent"
                        IconTheme{
                            anchors.fill : parent
                            icon : multitaskViewModel.getWindowIcon(wid)
                        }
                    }

                    Label {
                        id : caption
                        Layout.fillWidth : true
                        text : captionText
                        fontSizeMode : Text.HorizontalFit
                        elide : Text.ElideRight
                        color : "#FFFFFF"
                    }

                    TabletViewCloseButton {
                        id : preWindowCloseButton

                        states :  [
                            State {
                                name : "default"
                                PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-default.svg" }
                            },
                            State {
                                name : "hover"
                                PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-hover.svg" }
                            },
                            State {
                                name : "press"
                                PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-click.svg" }
                            }
                        ]

                        Component.onCompleted: {
                            preWindowCloseButton.setId(model.uuid)
                        }
                    }
                }//RowLayout captionRow

                Rectangle{
                    id : thumbBorder
                    height : Math.floor(preWinHeight)
                    width : Math.floor(preWinWidth + 8)
                    color : "transparent"
                    radius : tabletPreviewWindow.thumbnailRadius
                    border.color : "#FFFFFF"
                    border.width : Math.floor(borderWidth)

                    Rectangle{
                        id : thumbAnchorRoot
                        height : Math.floor(preWinHeight)
                        width : Math.floor(preWinWidth)
                        color : Qt.rgba(255,255,255, 0.5)
                        x : 4
                        y : 0
                        radius : tabletPreviewWindow.thumbnailRadius
                        property bool thumbClicked: false//记录是否是点击缩略图

                        Rectangle{
                            id : thumbMove
                            width : Math.floor(parent.width)
                            height : Math.floor(parent.height)
                            color : "transparent"
                            anchors.centerIn : parent
                            radius : tabletPreviewWindow.thumbnailRadius

                            states: [
                                State {
                                    name : "mousestate"
                                    when : thumbAnchorRoot.thumbClicked
                                    ParentChange {
                                        target : thumbAnchorRoot
                                        parent : rootView
                                    }
                                }
                            ]

                            Rectangle {
                                id:thumb
                                visible:true

                                function thumbWidth(){
                                    var factor = model.height / thumbMove.height;
                                    var width = model.width / factor;
                                    return Math.min(thumbMove.width, width);
                                }

                                function thumbHeight(){
                                    var factor = thumbMove.width * model.height
                                    var height = factor / model.width;
                                    return Math.floor(Math.min(thumbMove.height, height))
                                }

                                width : Math.floor(thumb.thumbWidth())
                                height : Math.floor(thumb.thumbHeight())
                                anchors.centerIn : parent
                                radius : tabletPreviewWindow.thumbnailRadius
                                color : "transparent"
                                clip : true
                                layer.enabled : multitaskViewManager.useOpenGL() ? true : false
                                layer.effect : OpacityMask{
                                    maskSource : Rectangle{
                                        width : Math.floor(thumb.width)
                                        height : Math.floor(thumb.height)
                                        radius : tabletPreviewWindow.thumbnailRadius
                                    }
                                }
                                KWinComponents.WindowThumbnailItem {
                                    anchors.centerIn : parent
                                    height : Math.floor(thumb.height)
                                    width : Math.floor(thumb.width)
                                    clip : true
                                    wId : wid
                                }
                            }
                        }
                    }
                }
            }
        }

    }

    DragHandler{
        id : windowDrag
        target : tabletPreviewWindow
        xAxis.enabled : false
        yAxis.maximum : tabletPreviewWindowMove.height / 10
        property int dragTime : 0
        property var beginTime : Date.now()

        onActiveChanged: {
            if (active){
                enlargeWindow.start()
                appWindowReturnBackAnimation.stop()
                beginTime = Date.now()
            }
            if (!active){
                var subTime = Date.now() - beginTime
                var subDistance = tabletPreviewWindowMove.y - tabletPreviewWindow.y
                var v = Math.abs(subDistance) / subTime;
                console.log("移动距离 ，窗口的三分之二" + subDistance ,  tabletPreviewWindow.height * 2 / 3)
                console.log("速度为: " + v)
                // 平均速度或者位移距离
                if ((v > 0.5 && subDistance > 0 ) || subDistance >= tabletPreviewWindow.height * 2 / 3){
                    appController.slideUpCloseWindow()
                }else{
                    appController.startResumeWindowAnimation()
                }
            }
        }
    }
}


