/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import "TabletController.js" as TabletController

Rectangle{
    id : tabletScrollViewArea
    width : 1920
    height : 1080
    color : "transparent"
    property ListModel appModel : null
    property var rootRect : null
    property var tempScreenIndex : null
    //调用全部清除按钮动效的时候需要用到
    property var appAreaController : null

    ScrollView{
        id : scrollView

        //更新滚动区域的大小，之所以这么做，是为了解决滚动区域增加鼠标事件后，滚动失效的问题
        function getContentWidth()
        {
            var maxColumn = Math.ceil(appModel.count / 2)
            var scrollViewContentWidth = appArea.width
            if (appModel.count <= 4)
            {
                scrollViewContentWidth = appArea.width
                return Math.floor(scrollViewContentWidth)
            }
            scrollViewContentWidth = maxColumn * preWinWidth + maxColumn * gridLayout.columnSpacing + 2 * gridLayout.leftPadding
            return Math.floor(scrollViewContentWidth)
        }

        x : 0
        y : 0
        contentWidth : Math.floor(scrollView.getContentWidth())
        contentHeight : Math.floor(appArea.height)
        width : Math.floor(rootRect.width)
        height: Math.floor(appArea.height)
        ScrollBar.horizontal.policy : ScrollBar.AlwaysOff
        ScrollBar.vertical.policy : ScrollBar.AlwaysOff

        Rectangle {
            id : scrollBackRect
            anchors.fill: parent
            color: "transparent"
            MouseArea{
                anchors.fill : parent
                onClicked : {
                    appAreaController.viewSpacingClickedController()
                    console.log("click and exit =============")
//                    appAreaController.closeMultitaskView()
                }
            }
            DragHandler{
                id : scrollBackDrag
                target : scrollBackRect
                xAxis.enabled : false
                yAxis.enabled : false
            }//windowDrag
        }

        Grid{
            id : gridLayout
            anchors.fill : parent
            anchors.centerIn : parent
            topPadding: Math.floor(scrollViewYStartPoint)
            columnSpacing : Math.floor(preWinVecSpace)
            rowSpacing : Math.floor(preWinHorSpace)
            leftPadding : Math.floor(scrollViewLeftSpace)
            rightPadding : Math.floor(scrollViewLeftSpace)
            rows : 2
            flow : Grid.TopToBottom
            transformOrigin : "TopLeft"

            //显示多任务视图时，每个Item在显示的时候执行的动效
            add : Transition {
                id : displayPreviewWinAnimation

                ParallelAnimation{
                    NumberAnimation {
                        property : "opacity"
                        duration : 900
                        easing.type : Easing.InOutQuart
                        from : 0
                        to : 1
                    }
                   NumberAnimation {
                       property : "x"
                       duration : 900
                       easing.type : Easing.InOutQuart
                       from : Math.floor(rootRect.width / 2 - 510/*窗口的宽度*/ / 2)
                   }
                   NumberAnimation {
                       property : "y"
                       duration : 900
                       easing.type : Easing.InOutQuart
                       from : Math.floor(rootRect.height)
                   }
                   NumberAnimation {
                       property : "scale"
                       from : 0
                       to : 1.0
                       duration : 450
                   }
                }
            }

            //关闭小窗口时，W补齐方式的动画
            move : Transition {
                NumberAnimation{
                    properties : "x,y";
                    easing.type : Easing.InOutQuart;
                    duration : 300
                }
            }


            Repeater{
                id : previewWindowRepeater
                model : appModel

                Rectangle{
                    id : appWindowBack
                    height : Math.floor(preWinHeight + iconToThumbSpacing + iconAndCloseButtonHeight)
                    width : Math.floor(preWinWidth)
                    color : "transparent"
                    radius : 14

                    TabletPreviewWindow {
                        id : appWindow
                        height : Math.floor(preWinHeight + iconToThumbSpacing + iconAndCloseButtonHeight)
                        width :  Math.floor(preWinWidth)
                        color : "transparent"
                        radius : 14
                        premodel : appModel
                        screenIndex : tempScreenIndex

                        states :  [
                            State {
                                name : "default"
                                PropertyChanges { target : appWindow; borderWidth : 0 }
                                PropertyChanges { target : appWindow; closeButtonVisble : false }
                            },

                            State {
                                name : "hover"
                                PropertyChanges { target : appWindow; borderWidth : 4 }
                                PropertyChanges { target : appWindow; closeButtonVisble : true }
                            }
                        ]

                        Connections{
                            target : allrootRepeater
                            function onQmlViewSpaceClicked() {
                                appWindow.appController.startSpaceClickedPreWindowAnimation()
                            }
                        }

                        Connections{
                            target : rootView
                            function onQmlClearAllAppWindow() {
                                appWindow.appController.startClearAllWindowAnimation()
                            }
                        }

                        Component.onCompleted: {
                            appWindow.setCaption(model.caption)
                            appWindow.setId(model.uuid)
                            appWindow.setPreViewNoScaledHeight(model.height)
                            appWindow.setPreViewNoScaledWidth(model.width)
                            appWindow.setPreViewNoScaledXCoordinate(model.x)
                            appWindow.setPreViewNoScaledYCoordinate(model.y)
                        }
                    }
                }
            }
        }
    }
}
