/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0

Rectangle{
    id : tabletViewClearAllWindowButton
    height : Math.floor(48 * tabletScaleFactor)
    width : language === "Chinese" ? Math.floor(126 * tabletScaleFactor) : Math.floor(184 * tabletScaleFactor)
    color :  Qt.rgba(255,255,255, 0.4)
    radius : 10
    anchors.verticalCenter : parent.verticalCenter
    anchors.verticalCenterOffset : Math.floor(screenRect.height / 2 - 48 * tabletScaleFactor -38 * tabletScaleFactor)
    anchors.horizontalCenter : parent.horizontalCenter
    anchors.horizontalCenterOffset : 0
    visible : tabletViewRootModel.count>0
    opacity : 0


    Rectangle{
        id : deleteIcon
        width : Math.floor(24 * tabletScaleFactor)
        height : Math.floor(24 * tabletScaleFactor)
        color : "transparent"
        x : Math.floor(27 * tabletScaleFactor)
        y : Math.floor(12 * tabletScaleFactor)
        Image {
            id : deleteImage
            anchors.fill : parent
            source : "qrc:/images/tablet_delete.svg"
        }
    }

    Label{
        id : deletText
        x : Math.floor(60 * tabletScaleFactor)
        y : Math.floor(10 * tabletScaleFactor)
        Layout.fillWidth : true
        text : qsTr("Clear") // 清除
        elide : Text.ElideRight
        font.pixelSize : 18 * tabletScaleFactor
        color : "#000000"
    }

    MouseArea{
        anchors.fill : parent
        hoverEnabled : true
        onEntered: {
            tabletViewClearAllWindowButton.color =  Qt.rgba(255,255,255, 0.6)
        }
        onPressed: {
            tabletViewClearAllWindowButton.color =  Qt.rgba(255,255,255, 0.8)
        }
        onExited: {
            tabletViewClearAllWindowButton.color =  Qt.rgba(255,255,255, 0.4)
        }

        onClicked: {
            console.log("全部清除被点击！")
            controller.startClearAllBtnDisappearAni()
            qmlClearAllAppWindow()
        }
    }

    function startClearAllBtnAppearAni()
    {
        clearAllWindowButtonAppearAni.start()
    }

    function startClearAllBtnDisappearAni()
    {
        clearAllWindowButtonDisappearAni.start()
    }

    //全部清除按钮出现的动画
    NumberAnimation {
       id : clearAllWindowButtonAppearAni
       target : tabletViewClearAllWindowButton
       properties : "opacity"
       from : 0
       to : 1
       duration : tabletViewRootModel.count >= 6 ? 5 * 100 + 300 : (tabletViewRootModel.count - 1) * 100 + 300
       easing.type : Easing.InOutQuart
    }

    //全部清除按钮消失的动画
    NumberAnimation {
       id : clearAllWindowButtonDisappearAni
       target : tabletViewClearAllWindowButton
       properties : "opacity"
       from : 1
       to : 0
       duration : 300
       easing.type : Easing.InOutQuart
    }
}
