/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import QtGraphicalEffects 1.0

Rectangle {
    id : closeButton
    visible : closeButtonVisble
    width : Math.floor(iconAndCloseButtonHeight)
    height : Math.floor(iconAndCloseButtonHeight)
    radius : 14
    color : "transparent"

    property var closeButtonController : new TabletController.CloseButtonController(multitaskViewModel, closeButton, multitaskViewManager, ["app"])
    property var buttonStateList : ["default", "hover", "press"];
    state : "default"
    property var imageSource : "qrc:/images/ukui-taskview-close-default.svg"
    property string wid : ""
    property alias closeButtonContainMouse: closeButtonMouseArea.containsMouse

    Image {
        id: closeWindow
        anchors.fill : parent
        source : imageSource
        sourceSize.width: Math.floor(parent.width)
        sourceSize.height: Math.floor(parent.height)
    }

    function setStatus(status) {
          if(buttonStateList.indexOf(status) < 0)
          {
              return;
          }
         if (status === state) {
             return
         } else {
             state = status;
         }
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function setPressStatus() {
        setStatus("press")
    }

    function setId(wId) {
        wid = wId;
    }

    MouseArea{
        id : closeButtonMouseArea
        anchors.fill : parent
        hoverEnabled : true
        onEntered: {
           closeButtonController.setHoverStatus()
        }

        onExited: {
            closeButtonController.setDefaultStatus()
            if(!preWindowMouseArea.containsMouse)
            {
                tabletPreviewWindow.setDefaultStatus()
            }
        }

        onPressed: {
           closeButtonController.setPressStatus()
        }

        onClicked: {
            tabletPreviewWindow.removePreviewWindow(wid)
        }
    }
}
