/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import org.kde.kwin 3.0 as KWinComponents

Repeater{
    id : allrootRepeater
    model : screencnt
    property int screencnt : multitaskViewModel.logicalScreenCount
    property var controller : new TabletController.Controller(multitaskViewModel, allrootRepeater, multitaskViewManager, ["none"])
    /*
        此处的focus不可以删除，因为后台在grabbedKeyboardEvent()函数中增加了setfocus()
        不增加focus:true前端无法接受键盘事件
        这是正向解决方法，其他方法非正向解决，不推荐！！！
    */
    focus: true

    Keys.onPressed: {
        if (event.key === Qt.Key_Escape) {
            controller.closeMultitaskView();
            return;
        }
        if (event.modifiers & Qt.MetaModifier) {
            if (event.key === Qt.Key_Tab) {
                controller.closeMultitaskView();
                return;
            }
        }
    }

    signal qmlAppWindowActivated()
    signal qmlAppWindowCreated(string id)
    signal qmlAppWindowRemoved(string id)
    signal qmlDeviceModeChanged(bool modeFlag)
    signal qmlOrientationChanged(string orientation)
    signal qmlScreenCountChanged(int oldCount,int newCount)
    signal qmlScreenSizeChanged()
    //后台发送qmlAppWindowRemoved，通过qmlAppWindowRemovedForward通知视图模块删除model内的数据
    signal qmlAppWindowRemovedForward(string id)
    signal qmlViewSpaceClicked()
    signal qmlStartClearAllBtnDisappearAni()


    // onQmlAppWindowActivated : {
    //     console.log("onQmlAppWindowActivated")
    //     allrootRepeater.controller.closeMultitaskView();
    // }

    onQmlAppWindowCreated: {
        console.log("onQmlAppWindowActivated")
        allrootRepeater.controller.closeMultitaskView();
    }

    onQmlDeviceModeChanged : {
        console.log("onQmlDeviceModeChanged")
        allrootRepeater.controller.closeMultitaskView();
    }

    onQmlAppWindowRemoved : {
        allrootRepeater.qmlAppWindowRemovedForward(id)
    }

    onQmlScreenCountChanged : {
        console.log("onQmlScreenCountChanged")
        allrootRepeater.controller.closeMultitaskView();
    }

    onQmlScreenSizeChanged : {
        console.log("onQmlScreenSizeChanged")
        allrootRepeater.controller.closeMultitaskView();
    }

    function viewSpacingClickedController()
    {
        qmlStartClearAllBtnDisappearAni()
        qmlViewSpaceClicked()
    }

    Rectangle{
        id : tabletViewMainWindow
        property var screenIndex : index
        property var screenRect : multitaskViewModel.getFullScreenGeometry(multitaskViewModel.getCurrentDesktop(),screenIndex)
        property string  orientation: multitaskViewModel.rotationMode
        //整体缩放比
        property var tabletScaleFactor : screenRect.width > screenRect.height ? screenRect.height / 1080 : screenRect.width / 1080
        //预览窗口宽高比
        property var preWinAspectRatio: screenRect.width / screenRect.height
        //图标和缩略图的间距
        property var iconToThumbSpacing : Math.floor(18 * tabletScaleFactor)
        //图标高度
        property var iconAndCloseButtonHeight : Math.floor(44 * tabletScaleFactor)
        //缩略图高度
        property var preWinHeight : Math.floor(320 * tabletScaleFactor)
        //缩略图宽度
        property var preWinWidth : Math.floor(preWinHeight * preWinAspectRatio)
        //预览窗口列间距
        property var preWinVecSpace : Math.floor(112 * tabletScaleFactor)
        //预览窗口行间距
        property var preWinHorSpace : Math.floor(56 * tabletScaleFactor)
        //滚动区域距离左侧和右侧的间距
        property var scrollViewLeftSpace : Math.floor(112 * tabletScaleFactor)
        //滚动区域高度
        property var scrollViewHeight : Math.floor(820 * tabletScaleFactor)
        //全部清除按钮x
        property var clearWindwoX : Math.floor(747 * tabletScaleFactor)
        //全部清除按钮y
        property var clearWindwoY : Math.floor(994 * tabletScaleFactor)
        //滚动区域y轴起始高度
        property var scrollViewYStartPoint : Math.floor(getCurrentPreWindowHeight())

        property ListModel tabletViewRootModel : ListModel{}

        property var language : multitaskViewModel.getCurrentLanguage()

        width : Math.floor(screenRect.width)
        height : Math.floor(screenRect.height)
        x : Math.floor(screenRect.x)
        y : Math.floor(screenRect.y)
        color : "black"

        //这里放置后台提供的桌面背景的Item
        KWinComponents.DesktopBackground {
            width : Math.floor(tabletViewMainWindow.width)
            height : Math.floor(tabletViewMainWindow.height)
            blurAlpha : 0.7
        }

        /**********************************************函数*********************************************/
        function getCurrentPreWindowHeight()
        {
            console.log("当前开始计算每一个预览窗口的尺寸")
            if ( orientation == "normal" ||  orientation == "upside-down" )
            {
                scrollViewYStartPoint = Math.floor(99 * tabletScaleFactor)
                preWinHeight = Math.floor(320 * tabletScaleFactor)
                tabletParentWindow.preWinWidth = Math.floor(preWinHeight * preWinAspectRatio)
                preWinVecSpace = Math.floor(112 * tabletScaleFactor)
                preWinHorSpace = Math.floor(56 * tabletScaleFactor)
                scrollViewLeftSpace = Math.floor(112 * tabletScaleFactor)
                scrollViewHeight = Math.floor(820 * tabletScaleFactor)
                clearWindwoX = Math.floor(747 * tabletScaleFactor)
                clearWindwoY = Math.floor(994 * tabletScaleFactor)
            }
            else if ( orientation == "left" ||  orientation == "right" )
            {
                scrollViewYStartPoint = Math.floor(196 * tabletScaleFactor)
                preWinHeight = Math.floor(498 * tabletScaleFactor)
                preWinWidth = Math.floor(preWinHeight * preWinAspectRatio)
                preWinVecSpace = Math.floor(96 * tabletScaleFactor)
                preWinHorSpace = Math.floor(68 * tabletScaleFactor)
                scrollViewLeftSpace = Math.floor(96 * tabletScaleFactor)
                scrollViewHeight = Math.floor(1196 * tabletScaleFactor)
                clearWindwoX = Math.floor(477 * tabletScaleFactor)
                clearWindwoY = Math.floor(1534 * tabletScaleFactor)
            }
        }

        //根窗口的鼠标事件y
        MouseArea{
            anchors.fill : parent
            onClicked: {
                controller.viewSpacingClickedController()
            }
        }

        TabletMultitaskView {
            id : tabletViewMain
            anchors.fill : parent
            color : Qt.rgba(0,0,0,0.35)
        }
    }
}

