/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0

Rectangle{
    id : tabletRootWindow
    anchors.fill : parent
    property var controller : new TabletController.Controller(multitaskViewModel, tabletRootWindow, multitaskViewManager, ["none"])
    property var appWindowCount : 0
    property var loadedWindowNum : 0

    /**********************************************函数*********************************************/

    function startNoWindowTipsAnimation()
    {
        console.log("启动无窗口动画")
        noWindowTipsAnimation.start()
    }
    function startSequentialLoadWindowTimer()
    {
        console.log(" 启动计时器")
        sequentialLoadWindowTimer.start()
    }

    function fetchWindowNum(){
        var appWindowList =  multitaskViewModel.getAppWindowList(multitaskViewModel.getCurrentDesktop(),tabletViewMainWindow.screenIndex)
        for (var i in appWindowList)
        {
            tabletRootWindow.appWindowCount = tabletRootWindow.appWindowCount+1
        }
        console.log("当前的窗口数量为",tabletRootWindow.appWindowCount )
        controller.initViewAnimation()
    }

    function sequentialLoadWindow() {
        var appWindowList =  multitaskViewModel.getAppWindowList(multitaskViewModel.getCurrentDesktop(),tabletViewMainWindow.screenIndex)
        if (loadedWindowNum < appWindowCount && loadedWindowNum < 6 )
        {
            var info = appWindowList[loadedWindowNum]
            tabletViewRootModel.append({"x": info.x, "y": info.y,
                              "width": info.width, "height": info.height,
                              "id": info.id,"minimized":info.minimized, "caption": info.caption, "uuid" : String(info.uuid)});
            loadedWindowNum = loadedWindowNum+1
            console.log("加载窗口中，目前已加载窗口数量为" + loadedWindowNum)

        }
        else
        {
            sequentialLoadWindowTimer.stop()
            console.log("计时器停止，窗口数量为： " + tabletViewRootModel.count)
        }
        if (loadedWindowNum >= 6)
        {
            sequentialLoadWindowTimer.stop()
            while(loadedWindowNum <  appWindowCount)
            {
                var info = appWindowList[loadedWindowNum]
                tabletViewRootModel.append({"x": info.x, "y": info.y,
                                   "width": info.width, "height": info.height,
                                   "id": info.id,"minimized":info.minimized, "caption": info.caption, "uuid" : String(info.uuid)});
                loadedWindowNum = loadedWindowNum + 1
            }
            console.log("sequentialLoadWindow的剩余窗口加载完毕！！！")

        }
        if (loadedWindowNum >= appWindowCount || appWindowCount <= 0)
        {
            console.log("全部清除按钮显示或者无窗口显示")
            sequentialLoadWindowTimer.stop()
            clearAllWindow.startClearAllBtnAppearAni()
        }
    }

    /**********************************************动画*********************************************/
    SequentialAnimation{
        id :  noWindowTipsAnimation
        //无窗口文字出现动画
        NumberAnimation {
            id : noWindowTipsAppearAni
            target : noWindowText
            property : "opacity"
            duration : 1000
            easing.type : Easing.InOutQuart
            from : 0
            to : 1
        }

        //无窗口文字消失动画
        NumberAnimation {
            id : noWindowTipsDisappearAni
            target : noWindowText
            property : "opacity"
            duration : 1000
            easing.type : Easing.InOutQuart
            from : 1
            to : 0
        }
        onFinished: {
            console.log("无窗口文字消失动画")
           controller.closeMultitaskView()
        }
    }


    //控制启动动效中，小窗口之间的间隔
    Timer {
        id : sequentialLoadWindowTimer;
        interval : 150;
        repeat : true
        running : false
        triggeredOnStart : false
        onTriggered: {
            tabletRootWindow.sequentialLoadWindow()
          }
    }

    //无应用时的窗口显示
    Text {
        id : noWindowText
        anchors.centerIn : parent
        font.bold : true
        font.pixelSize : 25
        color : "white"
        text : qsTr("No recent tasks") // 无应用开启
        visible : tabletViewRootModel.count == 0
        opacity : 0
    }

    MouseArea{
        anchors.fill : parent
        propagateComposedEvents: true

        onClicked: {
            mouse.accepted = false
        }
    }

    Component.onCompleted: {
        console.log("计算当前多任务视图应该显示的窗口数量")
        tabletRootWindow.fetchWindowNum();
    }
}
