/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller

Repeater {
    id : allroot
    model : screencnt
    focus: true
    Keys.enabled: true

    property int screencnt : multitaskViewModel.logicalScreenCount
    property var allcontroller : new Controller.Controller(multitaskViewModel, allroot, multitaskViewManager, ["none"])

    signal qmlAppWindowCreated(int visiblestatus)
    signal qmlAppWindowRemoved(int Id)
    signal qmlAppWindowActivated()
    signal qmlAppWindowDesktopChanged(int windowId, int oldDesktopIndex, int newDesktopIndex)
    signal qmlDesktopRemoved(int Id)
    signal qmlDesktopAppended(int Id)
    signal qmlDesktopMoved(int oldDesktop,int newDesktop)
    signal qmlCurrentDesktopChanged(int old,int newId)
    signal qmlScreenCountChanged(int oldCount,int newCount)
    signal qmlDeviceModeChanged(bool modeFlag)
    signal qmlScreenSizeChanged()

    Keys.onPressed : {
        if (event.key === Qt.Key_Escape) {
            allroot.allcontroller.closeMultitaskView();
            return;
        }
        if (event.modifiers & Qt.MetaModifier)
        {
            if (event.key === Qt.Key_Tab)
            {
                allroot.allcontroller.closeMultitaskView();
                return;
            }
        }
    }



    Rectangle{
        id : rootWindow
        width : rootWindow.screenRect.width
        height : rootWindow.screenRect.height
        x : rootWindow.screenRect.x
        y : rootWindow.screenRect.y
        color : "black"

        property var windowIndex : index
        property var screenRect : multitaskViewModel.getFullScreenGeometry(multitaskViewModel.getCurrentDesktop(),rootWindow.windowIndex)
        property int highlightDesktop : multitaskViewModel.getCurrentDesktop()
        property var currentTheme  : multitaskViewModel.getUKUIStyleName()
        property var closeButtonHoverColor : setItemColor()
        property var closeButtonDefaultColor : ""
        property var closeButtonClickColor : ""
        property var desktopCloseButtonDefaultColor : ""
        property var closeButtonHoverImage : "qrc:/images/application-exit-symbolic.svg"
        property var closeButtonDefaultImage : "qrc:/images/application-exit-symbolic-white.svg"
        property var closeButtonClickImage : "qrc:/images/application-exit-symbolic.svg"
        property var createDesktopButtonReceiveColor : ""
        property var createDesktopButtonDefaultColor : ""
        property var desktopMaskReceiveColor : ""
        property var desktopCaptionColor : ""
        property var desktopAreaColor : ""
        property var tabBorderColor : ""
        property ListModel appWindowModel : ListModel{}
        property ListModel desktopWindowModel : ListModel{}
        property var controller : new Controller.Controller(multitaskViewModel, rootWindow, multitaskViewManager, ["none"])

        Connections{
            target : allroot

             onQmlAppWindowRemoved : {
                for(var i = 0;i<rootWindow.appWindowModel.count;i++){
                    var info = rootWindow.appWindowModel.get(i);
                    if(info.id === Id) {
                        rootWindow.appWindowModel.remove(i);
                        rootWindow.updateDesktopWindowModel()
                        return;
                    }
                }
            }


            onQmlAppWindowDesktopChanged : {
                rootWindow.appRemoved(windowId);
            }

            onQmlCurrentDesktopChanged : {
                rootWindow.resetAppWindowModel(newId)
                rootWindow.highlightDesktop = newId
            }

            onQmlDesktopRemoved : {
                // 移除指定虚拟桌面，并更新虚拟桌面索引值
                rootWindow.desktopWindowModel.remove(Id - 1);

                if(rootWindow.desktopWindowModel.get(rootWindow.desktopWindowModel.count - 1).itemType === "workSpace")
                {
                    rootWindow.desktopWindowModel.insert(rootWindow.desktopWindowModel.count ,{value : rootWindow.desktopWindowModel.count, model : [], itemType: "newDesktop" });
                }

                for(var i = Id - 1; i < rootWindow.desktopWindowModel.count; ++i)
                {
                    var object = rootWindow.desktopWindowModel.get(i);
                    object.value = i+1
                }
                // 更新小窗口
                rootWindow.updateDesktopWindowModel()

                if (Id !== 1) {
                    Id -= 1
                }

                multitaskViewModel.setCurrentDesktop(Id);
                rootWindow.highlightDesktop  = multitaskViewModel.getCurrentDesktop()
                rootWindow.resetAppWindowModel(multitaskViewModel.getCurrentDesktop())
            }

            // desktopWindowModel中增加一个虚拟桌面
            onQmlDesktopAppended : {
                rootWindow.desktopWindowModel.insert(Id - 1,{value : Id, model : [], itemType: "workSpace" });

                if (rootWindow.desktopWindowModel.count === 16)
                {
                    rootWindow.desktopWindowModel.remove(Id)
                }

                multitaskViewModel.setCurrentDesktop(Id);
                rootWindow.highlightDesktop  = multitaskViewModel.getCurrentDesktop()
            }

            // 交换desktopWindowModel中的指定两项
            onQmlDesktopMoved : {
                rootWindow.desktopWindowModel.move(oldDesktop-1,newDesktop-1 , 1);
                multitaskViewModel.setCurrentDesktop(newDesktop);
                rootWindow.highlightDesktop = newDesktop
                // 交换后更新索引值
                for (var i = oldDesktop < newDesktop ? oldDesktop : newDesktop;
                i <= rootWindow.desktopWindowModel.count; ++i)
                {
                    var object = rootWindow.desktopWindowModel.get(i-1);
                    object.value = i;
                }
            }

            onQmlAppWindowCreated : {
                allroot.allcontroller.closeMultitaskView();
            }

            onQmlDeviceModeChanged : {
                console.log("onQmlDeviceModeChanged")
                allroot.allcontroller.closeMultitaskView();
            }

            // onQmlAppWindowActivated: {
            //     allcontroller.closeMultitaskView();
            // }

            onQmlScreenCountChanged : {
                allcontroller.closeMultitaskView();
            }

            onQmlScreenSizeChanged : {
                allcontroller.closeMultitaskView();
            }
        }

        KWinComponents.DesktopBackground {
            id : wholeDesktopBackground
            width : rootWindow.width
            height : rootWindow.height
            blurAlpha : 0.7
        }

        function setItemColor() {
            rootWindow.closeButtonDefaultColor = Qt.rgba(255,255,255, 0.15)
            if (rootWindow.currentTheme === "ukui-default" || rootWindow.currentTheme === "ukui-white"
                    || rootWindow.currentTheme === "ukui" || rootWindow.currentTheme === "ukui-light")
            {           
                rootWindow.desktopCloseButtonDefaultColor = Qt.rgba(38/255,38/255,38/255, 0.15)
                rootWindow.closeButtonHoverColor = Qt.rgba(255,255,255, 1)
                rootWindow.closeButtonClickColor = Qt.rgba(255,255,255, 0.75)
                rootWindow.createDesktopButtonReceiveColor = Qt.rgba(0, 0, 0, 0.25)
                rootWindow.createDesktopButtonDefaultColor = Qt.rgba(0, 0, 0, 0.15)
                rootWindow.desktopMaskReceiveColor = Qt.rgba(0, 0, 0, 0.25)
                rootWindow.desktopCaptionColor = Qt.rgba(0,0,0,1)
                rootWindow.desktopAreaColor = Qt.rgba(245/255, 245/255, 245/255,0.75)
                rootWindow.tabBorderColor = "#2c73c8"
                rootWindow.closeButtonHoverImage = "qrc:/images/application-exit-symbolic.svg"
                rootWindow.closeButtonDefaultImage = "qrc:/images/application-exit-symbolic.svg"
                rootWindow.closeButtonClickImage = "qrc:/images/application-exit-symbolic.svg"
            }
            else
            {
                rootWindow.desktopCloseButtonDefaultColor = Qt.rgba(255,255,255, 0.15)
                rootWindow.closeButtonHoverColor =  "#37373B"
                rootWindow.closeButtonClickColor = Qt.rgba(75/255,75/255,79/255, 1)
                rootWindow.createDesktopButtonReceiveColor = Qt.rgba(0, 0, 0, 0.1)
                rootWindow.createDesktopButtonDefaultColor = Qt.rgba(255, 255, 255, 0.15)
                rootWindow.desktopMaskReceiveColor = Qt.rgba(255, 255, 255, 0.25)
                rootWindow.desktopCaptionColor = Qt.rgba(255, 255, 255, 1)
                rootWindow.desktopAreaColor = Qt.rgba(35/255, 36/255, 38/255, 0.75)
                rootWindow.tabBorderColor = "#5fa6fb"
                rootWindow.closeButtonHoverImage = "qrc:/images/application-exit-symbolic-white.svg"
                rootWindow.closeButtonDefaultImage = "qrc:/images/application-exit-symbolic-white.svg"
                rootWindow.closeButtonClickImage = "qrc:/images/application-exit-symbolic-white.svg"
            }

        }


        //获取预览窗口的内容
        function resetAppWindowModel(deskindex) {
            var windowInfo = multitaskViewModel.getAppWindowList(deskindex,rootWindow.windowIndex)
            rootWindow.appWindowModel.clear();
            for(var i in windowInfo)
            {
                var info = windowInfo[i]
                rootWindow.appWindowModel.append({"x" : info.x, "y" : info.y,
                                                     "width" : info.width, "height" : info.height,
                                                     "id" : info.id,"minimized" : info.minimized, "caption" : info.caption, "uuid" : String(info.uuid)});
            }
        }

        //当拖拽窗口切换页面时，调用
        function appRemoved(wid){
            for(var i = 0;i<rootWindow.appWindowModel.count;i++){
                var info = rootWindow.appWindowModel.get(i);
                if(info.uuid === wid) {
                    rootWindow.appWindowModel.remove(i);
                    return;
                }
            }
        }

        // 重置desktopWindowModel信息，获取所有虚拟桌面的内容
        function resetDesktopWindowModel(){
            rootWindow.desktopWindowModel.clear()
            var desktopinfo = multitaskViewModel.getDesktopList();
            var desktopIndex = 0
            for (var i in desktopinfo)
            {
                // 获取虚拟桌面上的窗口显示情况
                var winModel = [];
                var infos = desktopinfo[i]
                var screenlist =infos.screenList
                var appList = screenlist[rootWindow.windowIndex].windowList
                for(var j in appList)
                {
                    var info = appList[j]

                    winModel.push({"x": info.x, "y": info.y,
                                      "width": info.width, "height": info.height,
                                      "id": info.id,"minimized": info.minimized,
                                      "caption": info.caption,"uuid" : String(info.uuid)});
                }
                rootWindow.desktopWindowModel.append({ value: infos.index, model: winModel, itemType: "workSpace"});
                desktopIndex = infos.index
            }
            if (desktopWindowModel.count < 15) {
                var desktopModel = [];
                rootWindow.desktopWindowModel.append({ value: desktopIndex + 1, model: desktopModel, itemType: "newDesktop"});
            }

        }

        // 更新全部desktopWindowModel信息,更新某一个虚拟桌面的内容
        function updateDesktopWindowModel(){
            var desktopinfo = multitaskViewModel.getDesktopList();

            for (var i in desktopinfo)
            {
                rootWindow.desktopWindowModel.get(i).model.clear();
                var infos = desktopinfo[i]
                var screenlist =infos.screenList
                var appList = screenlist[rootWindow.windowIndex].windowList

                for(var j in appList)
                {
                    var info = appList[j]
                    rootWindow.desktopWindowModel.get(i).model.append({"x": info.x, "y": info.y,
                                                                          "width": info.width, "height": info.height,
                                                                          "id": info.id,"minimized":info.minimized,
                                                                          "caption": info.caption, "uuid" : String(info.uuid)});
                }
            }
        }

        PCWindowViewFrame{
            windowIndex : index
            anchors.fill : parent
        }

        Component.onCompleted : {
            resetAppWindowModel(multitaskViewModel.getCurrentDesktop())
            resetDesktopWindowModel()
        }
    }
}
